{
    "version" : 1,
    "key" : {
        "algorithm" : "RSA",
        "data" : "-----BEGIN PUBLIC KEY-----\nMIGJAoGBALQOcgxhhV0XiHELKYdG587Tup261qQ3ahAGPuifZvxHXTq+GgulEyXiovwrVjpz7rKXn+16HgspLHpp5agv0WsSn6k2MnQGk5RFXuilbFr\/C1rEX2X7uXlUXDMpsriKFndoB1lz9P3E8FkM5ycG84hejcHB+R5yzDa4KbGeOc0tAgMBAAE=\n-----END PUBLIC KEY-----",
        "mode" : "ECB",
        "padding" : "PKCS1",
        "digest" : "SHA256"
    },
    "seed" : "moki",
    "fingerprint" : "ld68TnzYqzFQMxeJ6N+aZa2jRf9d4zVx4BUiBlmur67ne8YZF08plhCiIhfyYDIwwW7KLaAHvK8gJbp0pPIzLR4bhzu6zRpDLzUQsq6bXgMp+WAiZtFm6IHWNUwUEYcr3iSvTn5L1HunRt7kBglEjv8RKtbNcK0t1Xto375kMlo="
}
