{
    "algorithm" : "RSA",
    "data" : "-----BEGIN RSA PRIVATE KEY-----\nMIICXQIBAAKBgQC0DnIMYYVdF4hxCymHRufO07qdutakN2oQBj7on2b8R106vhoLpRMl4qL8K1Y6c+6yl5\/teh4LKSx6aeWoL9FrEp+pNjJ0BpOURV7opWxa\/wtaxF9l+7l5VFwzKbK4ihZ3aAdZc\/T9xPBZDOcnBvOIXo3Bwfkecsw2uCmxnjnNLQIDAQABAoGADi5wFaENsbgTh0HHjs\/LHKto8JjhZHQ33pS7WjOJ1zdgtKp53y5sfGimCSH5q+drJrZSApCCcsMWrXqPO8iuX\/QPak72yzTuq9MEn4tusO\/5w8\/g\/csq+RUhlLHLdOrPfVciMBXgouT8BB6UMa0e\/g8K\/7JBV8v1v59ZUccSSwkCQQD67yI6uSlgy1\/NWqMENpGc9tDDoZPR2zjfrXquJaUcih2dDzEbhbzHxjoScGaVcTOx\/Aiu00dAutoN+Jpovpq1AkEAt7EBRCarVdo4YKKNnW3cZQ7u0taPgvc\/eJrXaWES9+MpC\/NZLnQNF\/NZlU9\/H2607\/d+Xaac6wtxkIQ7O61bmQJBAOUTMThSmIeYoZiiSXcrKbsVRneRJZTKgB0SDZC1JQnsvCQJHld1u2TUfWcf3UZH1V2CK5sNnVpmOXHPpYZBmpECQBp1hJkseMGFDVneEEf86yIjZIM6JLHYq2vT4fNr6C+MqPzvsIjgboJkqyK2sLj2WVm3bJxQw4mXvGP0qBOQhQECQQCOepIyFl\/a\/KmjVZ5dvmU2lcHXkqrvjcAbpyO1Dw6p2OFCBTTQf3QRmCoys5\/dyBGLDhRzV5Obtg6Fll\/caLXs\n-----END RSA PRIVATE KEY-----",
    "mode" : "ECB",
    "padding" : "PKCS1",
    "digest" : "SHA256"
}
